/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2022 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.18.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
File    : RTOSInit_MSP430F5659.c
Purpose : Initializes and handles the hardware for embOS
*/

#include "RTOS.h"
#include <msp430f5659.h>

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/*********************************************************************
*
*       System tick settings
*/
#define OS_TIMER_FREQ    (4000000u)
#define OS_TICK_FREQ     (1000u)
#define OS_INT_FREQ      (OS_TICK_FREQ)
#define OS_TIMER_RELOAD  ((OS_TIMER_FREQ / OS_INT_FREQ) - 1u)

/*********************************************************************
*
*       embOSView settings
*/
#ifndef   OS_VIEW_IFSELECT
  #define OS_VIEW_IFSELECT  OS_VIEW_DISABLED
#endif

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  #include "BSP_UART.h"
  #define OS_UART      (0u)
  #define OS_BAUDRATE  (9600u)
#endif

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
/*********************************************************************
*
*       _OS_OnRX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static void _OS_OnRX(unsigned int Unit, unsigned char c) {
  OS_USE_PARA(Unit);
  OS_COM_OnRx(c);
}

/*********************************************************************
*
*       _OS_OnTX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static int _OS_OnTX(unsigned int Unit) {
  OS_USE_PARA(Unit);
  return (int)OS_COM_OnTx();
}
#endif

/*********************************************************************
*
*       _OS_GetHWTimerCycles()
*
*  Function description
*    Returns the current hardware timer count value.
*
*  Return value
*    Current timer count value.
*/
static unsigned int _OS_GetHWTimerCycles(void) {
  return TA0R;
}

/*********************************************************************
*
*       _OS_GetHWTimer_IntPending()
*
*  Function description
*    Returns if the hardware timer interrupt pending flag is set.
*
*  Return value
*    == 0: Interrupt pending flag not set.
*    != 0: Interrupt pending flag set.
*/
static unsigned int _OS_GetHWTimer_IntPending(void) {
  return (TA0CCTL0 & (1u << 0));
}

/*********************************************************************
*
*       _InitClocks()
*
*  Function description
*    Clock init.
*/
static void _InitClocks(void) {
#if (OS_DEBUG != 0)
  //
  // Output SMCLK (=XT2) on P3.4
  //
  P3DIR |= BIT4;
  P3SEL |= BIT4;
#endif
  //
  // Initialize XT2
  //
  P7SEL   |= BIT2 | BIT3;  // Select XT2 pins for input
  UCSCTL6 &= ~XT2OFF;      // Enable XT2
  UCSCTL3 |= SELREF_2;     // FLLref = REFO
  UCSCTL4 |= SELA_2;       // ACLK=REFO,SMCLK=DCO,MCLK=DCO
  //
  // Wait until clocks are stable
  //
  do {
    UCSCTL7 &= ~(XT2OFFG | XT1LFOFFG | XT1HFOFFG | DCOFFG);  // Clear XT2,XT1,DCO fault flags
    SFRIFG1 &= ~OFIFG;                                       // Clear fault flags
  } while (SFRIFG1 & OFIFG);                                 // Test oscillator fault flag
  UCSCTL6 &= ~XT2DRIVE0;                                     // Decrease XT2 Drive according to expected frequency
  UCSCTL4 |= SELS_5 | SELM_5;                                // SMCLK=MCLK=XT2
}

/*********************************************************************
*
*       OS_ISR_Tick()
*
*  Function description
*    This is the hardware timer exception handler.
*/
#pragma vector=TIMER0_A0_VECTOR
__interrupt void OS_ISR_Tick(void) {
  OS_INT_Enter();
  OS_TICK_Handle();
  OS_INT_Leave();
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*  Function description
*    Initialize the hardware required for embOS to run.
*/
void OS_InitHW(void) {
  WDTCTL   = WDTPW + WDTHOLD;  // Stop watchdog timer
  _InitClocks();               // Init clocks
  OS_INT_IncDI();
  //
  // Initialize timer for embOS
  //
  TA0CTL   = 0                 // Reset Timer_A5, division 1
           | (1 << 2)          // Clear timer
           | (2 << 8)          // Use SMCLK as timer clock
           ;
  TA0CCR0  = OS_TIMER_RELOAD;  // Set to 1ms
  TA0CCTL0 = 0                 // Initialize capture control
           | (1 << 4);         // Enable compare interrupt
  TA0CTL  |= (1 << 4);         // Start timer in UP-Mode
  //
  // Inform embOS about the timer settings
  //
  {
    OS_SYSTIMER_CONFIG SysTimerConfig = {OS_TIMER_FREQ, OS_INT_FREQ, OS_TIMER_UPCOUNTING, _OS_GetHWTimerCycles, _OS_GetHWTimer_IntPending};
    OS_TIME_ConfigSysTimer(&SysTimerConfig);
  }
  //
  // Initialize communication for embOSView
  //
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Init(OS_UART, OS_BAUDRATE, BSP_UART_DATA_BITS_8, BSP_UART_PARITY_NONE, BSP_UART_STOP_BITS_1);
  BSP_UART_SetReadCallback(OS_UART, _OS_OnRX);
  BSP_UART_SetWriteCallback(OS_UART, _OS_OnTX);
#endif
  OS_INT_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*  Function description
*    This code is executed whenever no task, software timer, or
*    interrupt is ready for execution.
*
*  Additional information
*    The idle loop does not have a stack of its own, therefore no
*    functionality should be implemented that relies on the stack
*    to be preserved.
*/
void OS_Idle(void) {    // Idle loop: No task is ready to execute
  while (1) {           // Nothing to do ... wait for interrupt
    #if (OS_DEBUG == 0)
                        // Switch CPU into sleep mode
    #endif
  }
}

/*********************************************************************
*
*       Optional communication with embOSView
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_COM_Send1()
*
*  Function description
*    Sends one character.
*/
void OS_COM_Send1(OS_U8 c) {
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Write1(OS_UART, c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_DISABLED)
  OS_USE_PARA(c);          // Avoid compiler warning
  OS_COM_ClearTxActive();  // Let embOS know that Tx is not busy
#endif
}

/*********************************************************************
*
*       __low_level_init()
*
*  Function description
*    Called from IAR startup code.
*    Used to initialize hardware like PLL as early as possible.
*/
int __low_level_init(void) {
  WDTCTL = WDTPW + WDTHOLD;  // Stop watchdog timer
  _InitClocks();             // Init clocks
  return 1;
}

/*************************** End of file ****************************/
